/* Current issues and would-be-but-don't-care issues:
	The weight (T) tag doesn't handle quoted properties quite as it should
	The appendage (P) tag doesn't handle modifiers on quoted properties (is this a problem?)
	Need a better word for w` in appendages tag (P)
	Sometimes a space must follow the species tag in order to distinguish it from the next tag (eg Dw[H]Gm)
	Many of the tags ignore quoted properties when they are obvious from the tags column
	Custom properties on the original DC must be quoted ala the DC2
	Experimental original DC tags are not supported
	Some idiosyncracies with the skin-type none
	The diet tag has some weird bugs
	*** I'm assuming skin-type none is supposed to be Sk and not S
	*/

/* dcd2.5-a
	Supports DC V2.5
	*/

/* V.851
	Minor bug fixed involving bracketed options and crashing (ie V[Ice] would see the capital I and think it a new tag and then crash because the bracket didn't close ;)
	Now the Windows tray version won't take out the copying program and every other clipboard viewer in the system if it crashes
	*/

/* V.85
    Shapeshifter tags can now have {} characteristics (although the output format is not too friendly)
	Supports DC V2.4
	*/

/* V.81
	First public release
	Minor bug fixes over V.8
	*/

/* V.8
	Every saga has a beginning...  And, actually, this version wasn't it.
	But it might as well have been
	Totally rewritten from the badly-coded barely-working original version of the DC2 decoder I wrote, only a few of the large data tables were kept.
	Known to support DC V2.3 and the original DC, should support all V2.* up to and including 2.3
	*/

// Dragon Code Decoder
// Version: 0.851
// COPYRIGHT 2000 Amthrax Arlan

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

// Provide strnicmp to Linux
#ifndef WIN32
int strnicmp(char *s1, char *s2, int len)
{
	int i;
	for (i=0;i<len;i++)
		if (toupper(s1[i])!=toupper(s2[i]))
			return 1;
		return 0;
}
#endif

typedef struct
{
	char tag[3];
	int misc;
	void (*handler)(char tag[], char *out, int misc);
} DCTagHandler;

#define MAX_HANDLERS	27
DCTagHandler handlers[MAX_HANDLERS];
int numhandlers;

void DCDecode(char in[], char *out, int forcever=0);

int AddHandler(char tag[], int misc, void (*handler)(char tag[], char *out, int misc))
{
	if (numhandlers==MAX_HANDLERS)
		return -1;
	strcpy(handlers[numhandlers].tag,tag);
	handlers[numhandlers].misc=misc;
	handlers[numhandlers].handler=handler;
	numhandlers++;
	return 0;
}

typedef struct
{
	char key[20];
	char out[256];
} DictionaryEntry;

int Dictionary(DictionaryEntry d[], char key[], char absolute)
{
	int i;
	i=0;
	while (strlen(d[i].key))
	{
		if (absolute)
		{
			if (strcmp(key,d[i].key)==0)
				return i;
		}  else {
			if (strncmp(key,d[i].key,strlen(d[i].key))==0)
				return i;
		}
		i++;
	}
	if (key[0]=='\"')
	{
		unsigned int pos;
		// Muhaha!
		strcpy(d[i+1].key,"\"");
		pos=1;
		while (pos<strlen(key)&&key[pos]!='\"')
		{
			d[i+1].key[strlen(d[i+1].key)+1]=0;
			d[i+1].key[strlen(d[i+1].key)]=key[pos++];
		}
		strcat(d[i+1].key,"\"");
		strcpy(d[i+1].out,d[i+1].key);
		return i+1;
	}
	return i;
}

void DC2SpeciesSub(char tag[], char *out)
{
	DictionaryEntry st[] = { // Dragons
		{"Dss","Sea Serpent Dragon"},
		{"Dsf","Fire Serpent Dragon"},
		{"Da","Amphitere Dragon"},
		{"Dc","Draconid"},
		{"Dd","Dragonette"},
		{"De","Eastern Dragon"},
		{"Df","Faerie Dragon"},
		{"Dh","Hydra Dragon"},
		{"Di","Dimar Dragon"},
		{"Dl","Dracolich"},
		{"Dp","Pernese Dragon"},
		{"Dr","Turtle Dragon"},
		{"Ds","Serpent Dragon"},
		{"Dt","Tarrasque Dragon"},
		{"Du","Pseudodragon"},
		{"Dv","Wyvern"},
		{"Dw","Western Dragon"},
		{"Dy","Wyrn"},
		{"D","Dragon"},
		// Humanoids
		{"Hew","Wood Elf"},
		{"Ha","Ape"},
		{"Hd","Dwarf"},
		{"He","Elf"},
		{"Hf","Fairy"},
		{"Hi","Giant"},
		{"Hg","Gnome"},
		{"Hh","Hobit"},
		{"Hk","Kender"},
		{"Hy","Nymph"},
		{"Ht","Troll"},
		{"Hw","Wolfman"},
		{"H","Humanoid"},
		// Amphibains
		{"Af","Frog"},
		{"An","Newt"},
		{"As","Salamander"},
		{"At","Toad"},
		{"A","Amphibian"},
		// Birds
		{"Bc","Crow"},
		{"Be","Eagle"},
		{"Bh","Hawk"},
		{"Bp","Phoenix"},
		{"Br","Raven"},
		{"B","Birds"},
		// Crustaceans
		{"Cc","Crab"},
		{"Cl","Lobster"},
		{"Cs","Shrimp"},
		{"C","Crustacean"},
		// Dinosaurs
		{"Sa","Allosaurs"},
		{"Sc","Triceratops"},
		{"Sp","Apatosaurus"},
		{"Ss","Stegosaurus"},
		{"St","Tyrannosaurus"},
		{"Sv","Velociraptor"},
		{"S","Dinosaur"},
		// Extraterrestrial
		{"Ed","Dalek"},
		{"Et","Tibble"},
		{"E","Extraterrestrial"},
		// Fish
		{"Ffg","Goldfish"},
		{"Fft","Trout"},
		{"Fh","Sea Horse"},
		{"Ff","Freshwater Fish"},
		{"Fs","Shark"},
		{"F","Fish"},
		// Insects
		{"Ia","Ant"},
		{"Ib","Beetle"},
		{"If","Fly"},
		{"Il","Locust"},
		{"Im","Moth"},
		{"Iu","Butterfly"},
		{"I","Insects"},
		// Legends
		{"Lr","Gargoyle"},
		{"Ll","Gremlin"},
		{"Lg","Griffin"},
		{"Ln","Manitcore"},
		{"Lm","Mermaid/Merman"},
		{"Lf","Salamander (Lengendary)"},
		{"Ls","Sprite"},
		{"Lt","Treant/Treefolk"},
		{"Lu","Unicorn"},
		{"L","Lengend"},
		// Mammals
		{"Mfps","Snow Leopard"},
		{"Mcd","Domestic Dog"},
		{"Mcf","Fox"},
		{"Mcw","Wolve"},
		{"Mfb","Black Panther"},
		{"Mfc","Cheetah"},
		{"Mfd","Domestic Cat"},
		{"Mfp","Leopard"},
		{"Mfl","Lion"},
		{"Mfx","Lynx"},
		{"Mfa","Panther"},
		{"Mfu","Puma"},
		{"Mft","Tiger"},
		{"Mmg","Gibbon"},
		{"Mpf","Ferret"},
		{"Mpm","Mink"},
		{"Mrb","Bat"},
		{"Mrg","Gerbil"},
		{"Mrh","Hamster"},
		{"Mrm","Mice"},
		{"Mrr","Rat"},
		{"Mrs","Squirrel"},
		{"Mwd","Dolphin"},
		{"Mwk","Killer Whale"},
		{"Mwp","Porpoise"},
		{"Ma","Bats"},
		{"Mb","Bear"},
		{"Mc","Canine"},
		{"Mf","Feline"},
		{"Mh","Horse"},
		{"Mm","Monkey"},
		{"Mp","Polecat"},
		{"Mr","Rodent"},
		{"Mw","Cetacean (whale)"},
		{"M","Mammal"},
		// Molluscs
		{"Oc","Cuttlefish"},
		{"Ol","Limpet"},
		{"Oo","Octopuses"},
		{"Oy","Oyster"},
		{"Os","Snail"},
		{"O","Mullusc"},
		// Mythical
		{"Yc","Centaur"},
		{"Yy","Cyclopse"},
		{"Yg","Golem"},
		{"Yh","Hellhound"},
		{"Ym","Minotaur"},
		{"Yp","Pegasus"},
		{"Yt","Satyr"},
		{"Ys","Sphinx"},
		{"Y","Mythical"},
		// Plants
		{"Pta","Ash Tree"},
		{"Pte","Elm Tree"},
		{"Pto","Oak Tree"},
		{"Pc","Cactus"},
		{"Pf","Fungi"},
		{"Pt","Tree"},
		{"P","Plant"},
		// Reptiles
		{"Rnf","Fire Skink"},
		{"Ra","Alligator/Crocodile"},
		{"Rc","Chameleon"},
		{"Rg","Gecho"},
		{"Rk","Komodo Dragon"},
		{"Rl","Lizard"},
		{"Rn","Skink"},
		{"Rs","Snake"},
		{"Rt","Turtle"},
		{"R","Reptile"},
		// Spirits
		{"Qa","Angel"},
		{"Qd","Devil/Demon"},
		{"Qg","Ghost"},
		{"Qi","Imp"},
		{"Qp","Polterqeist"},
		{"Qs","Spectre"},
		{"Qw","Will-o-the-wisps"},
		{"Q","Spirit"},
		// Undead
		{"Ug","Ghoul"},
		{"Uv","Vampire"},
		{"Uz","Zombie"},
		{"U","Undead"},
		// Shapechangers
		{"~","Shapechanger"},
		// Misc
		{"?","Don't know"},
		{"",""},
		{"",""},
		};
	int ret;
	unsigned int pos;
	unsigned int i;

	ret=Dictionary(st,tag,0);
	if (strlen(st[ret].key))
	{
		strcat(out,st[ret].out);
		pos=strlen(st[ret].key);
		// Is there more to the tag?
modcheck:
		if (pos<strlen(tag))
		{
			// Is it a shapeshifter tag?
			if (strcmp(st[ret].key,"~")==0)
			{
				int found;
				// If there is a following "/", then it is between two things
				found=0;
				for (i=0;i<strlen(tag);i++)
					if (tag[i]=='/')
						found=0;
				if (found)
					strcat(out," between ");
				else
					strcat(out," in the form of a ");
				DC2SpeciesSub(&tag[pos],out);
			} else if (tag[pos]=='/') {		// Shapeshifter "and" fudge
				strcat(out," and ");
				DC2SpeciesSub(&tag[pos+1],out);
			} else if (tag[pos]=='?') {		// Unknown
				strcat(out," of unknown type");
			} else if (tag[pos]=='~') {		// Typed shapeshifter
				strcat(out," shapechanger");
			} else if (tag[pos]=='^') {		// Shaped
				strcat(out," with ");
				DC2SpeciesSub(&tag[pos+1],out);
				strcat(out," features");
			} else if (tag[pos]=='[') {		// In form
				strcat(out," in ");
				DC2SpeciesSub(&tag[pos+1],out);
				strcat(out," form");
			} else if (tag[pos]==']') {		// So it doesn't complain
				pos++;
				goto modcheck;		// Don't shoot me!
			} else if (tag[pos]=='{') {		// Form properties
				char str[256];
				int spos;
				// Find closing brace
				spos=0;
				pos++;
				while (tag[pos]!='}'&&spos<256&&pos<strlen(tag))
					str[spos++]=tag[pos++];
				strcat(out,":\n{\n");
				DCDecode(str,out,2);
				strcat(out,"\n}\n");
				goto modcheck;
			} else if (tag[pos]=='}') {		// Again, so it doesn't complain
				pos++;
				goto modcheck;
			} else if (tag[pos]=='+') {		// Cross
				strcat(out," crossed with ");
				DC2SpeciesSub(&tag[pos+1],out);
			} else if (tag[pos]=='\"') {	// Custom
				strcat(out," \"");
				pos++;
				while (pos<strlen(tag)&&tag[pos]!='\"')
				{
					out[strlen(out)+1]=0;
					out[strlen(out)]=tag[pos++];
				}
				strcat(out,"\"");
				pos++;
				if (pos<strlen(tag))
					goto modcheck;
			} else if (tag[pos]==' ') {	// Sometimes these sneek in
				pos++;
				if (pos<strlen(tag))
					goto modcheck;
			} else {
				strcat(out,"<Unrecognized modifier>");
			}
		}
	} else {
		strcat(out,"<Unrecognized species>");
	}
}

void DC2Species(char tag[], char *out)
{
	strcat(out,tag);
	strcat(out,"\tSpecies: ");
	DC2SpeciesSub(tag,out);
	strcat(out,"\n");
}

int DCSpeciesSub(char tag[], char *out)
{
	DictionaryEntry st[]={
		{"D","Dragon"},
		{"A","Angelic being"},
		{"C","Cat/felinoid"},
		{"Do","Dolphin"},
		{"Dw","Dwarf"},
		{"E","Elf"},
		{"e","Elemental"},
		{"F","Fox"},
		{"f","Fish"},
		{"Go","Goblin"},
		{"G","Gryphon"},
		{"H","Human"},
		{"M","Merfolk"},
		{"N","Demon"},
		{"O","Orc"},
		{"P","Pegasus"},
		{"R","Roc"},
		{"U","Unicorn"},
		{"V","Vampire"},
		{"W","Wolf"},
		{"X","Phoenix"},
		{"?","Shapeshifter"},
		{"!","Unknown"},
		{"",""},
		{"",""}};
	int ret;
	unsigned int pos;

	pos=0;
	while(pos<strlen(tag))
	{
		switch(tag[pos])
		{
		case '/':
			strcat(out," and ");
			pos++;
			break;
		case '(':
			strcat(out," in ");
			pos++;
			pos+=DCSpeciesSub(&tag[pos],out);
			strcat(out," form");
			pos++;
			break;
		case ')':
			return pos;
		default:
			ret=Dictionary(st,&tag[pos],0);
			if (strlen(st[ret].key))
			{
				strcat(out,st[ret].out);
				pos+=strlen(st[ret].key);
			} else {
				strcat(out,"<Unrecognized modifier>");
				pos++;
			}
			break;
		}
	}
	return pos;
}

void DCSpecies(char tag[], char *out)
{
	strcat(out,tag);
	strcat(out,"\tSpecies: ");
	DCSpeciesSub(tag,out);
	strcat(out,"\n");
}

unsigned int SimpleTag(char tag[], char *out, char pppb[], char ppp[], char pp[], char p[], char x[], char m[], char mm[], char mmm[], char mmmb[], char v[])
{
	char temp[256];
	if (!strcmp(tag,"+++!"))
		strcpy(temp,pppb);
	else if (!strcmp(tag,"+++"))
		strcpy(temp,ppp);
	else if (!strcmp(tag,"++"))
		strcpy(temp,pp);
	else if (!strcmp(tag,"+"))
		strcpy(temp,p);
	else if (!strcmp(tag,""))
		strcpy(temp,x);
	else if (!strcmp(tag,"-"))
		strcpy(temp,m);
	else if (!strcmp(tag,"--"))
		strcpy(temp,mm);
	else if (!strcmp(tag,"---"))
		strcpy(temp,mmm);
	else if (!strcmp(tag,"---!"))
		strcpy(temp,mmmb);
	else if (!strcmp(tag,"~"))
		strcpy(temp,v);
	else if (!strcmp(tag,"?"))
		strcpy(temp,"Unknown");
	else
		return 0;
	strcat(out,temp);
	if (strlen(tag))
		return strlen(tag);
	return 1;
}

unsigned int SimpleTagRel(char tag[], char *out, char pppb[], char ppp[], char pp[], char p[], char x[], char m[], char mm[], char mmm[], char mmmb[], char v[])
{
	if (!strncmp(tag,"+++!",4))
	{
		strcat(out,pppb);
		return 4;
	}
	else if (!strncmp(tag,"+++",3))
	{
		strcat(out,ppp);
		return 3;
	}
	else if (!strncmp(tag,"++",2))
	{
		strcat(out,pp);
		return 2;
	}
	else if (!strncmp(tag,"+",1))
	{
		strcat(out,p);
		return 1;
	}
	else if (!strncmp(tag,"---!",4))
	{
		strcat(out,mmmb);
		return 4;
	}
	else if (!strncmp(tag,"---",3))
	{
		strcat(out,mmm);
		return 3;
	}
	else if (!strncmp(tag,"--",2))
	{
		strcat(out,mm);
		return 2;
	}
	else if (!strncmp(tag,"-",1))
	{
		strcat(out,m);
		return 1;
	}
	else if (!strncmp(tag,"~",1))
	{
		strcat(out,v);
		return 1;
	}
	else if (!strncmp(tag,"?",1))
	{
		strcat(out,"Unknown");
		return 1;
	}
	else
	{
		strcat(out,x);
		return 0;
	}
}

void DC2SimpleHandler(char tag[], char *out, int misc)
{
	char info[5][11][256] = {{"Width: ",
		"'I am Athelind! My belly is now several galaxies wide ... while I'm only a few hundred feet long!'",
		"'Planets have been known to crack in half with my arrival!'",
		"'My digestion of food has been known to cause earthquakes.'",
		"'I move by rolling. Flying has always been an effort for me.'",
		"'What can I say ... I'm normal, except for a few feasts here or there.'",
		"'I'm slightly on the slim side!'",
		"'Ever heard of serpentine?'",
		"'Whoah! Whaddaya mean I look like a long string with wings?'",
		"'I'm one-dimensional - all length and no width or depth. Just one long super-string!'",
		"Variable - 'My girth depends on what I've just eaten!'"},
		{"Hoard-Size: ",
		"'Governments quake when they hear I'm going to sell!'",
		"'I really can't tell when someone steals an item!'",
		"'I'm comfortable, but I wouldn't mind some more.'",
		"'I not down to your last copper, but...'",
		"'I've got my own lair, unfurnished.'",
		"'The bank and I own my lair.'",
		"'The lair is rented, and cramped.'",
		"'No money, no lair, at least I'm not starving!'",
		"'Look on the bright side. At least I've got friends!'",
		"Variable"},
		{"Monetary-Philosophy: ",
		"'All things of value gravitate towards me. When I've got my claws on them I never let go!'",
		"'The coinage has my likeness on it. I own the whole kingdom!'",
		"'Besides humans, I kill other dragons for their wealth.'",
		"'Investments in human businesses. I do repossessions \"CHOMP\" personally!'",
		"'Take your hands off my hoard or I'll take your hands.'",
		"'Bought up the weapons smiths with my savings. At least there's no more dragon-proof armour or vorpal swords.'",
		"'Some thief stole most of it whilst I was out hunting hobbits!'",
		"'I'm a philanthropist. I gave it all to the poor!'",
		"'I eschew all things of value. I don't need them! Whatever I find I give away.'",
		"Variable"},
		{"Social-Life: ",
		"'I have made a determined effort to advertise the fact that I'm not a human. My close friends have got used to explaining me to strangers to save them time. I'm so militant about my true identity that even the men in white coats were impressed!'",
		"'My friends' parents all know. I use my on-line name in front of total strangers regularly. I've received letter from people I've never met asking questions about how to deal with being non-human.'",
		"'All of my friends know. I told my parents instead of waiting for them to find out. The local cybercaf's manager calls me by my on-line name.'",
		"'Most of my friends know. I'd tell my parents if I thought they'd take it well. I've used my on-line name in public on occasion.'",
		"'A few of my friends know. I'm glad my parents have never asked me about this. I'm rather low-key but don't dodge the subject.'",
		"'I don't want to be thought crazy in human life. If the subject came up, I'd tell my closest friends only.'",
		"'It's something that would make me rather uncomfortable to have let out. I'm open on-line but try to keep my human side untraceable, and vice versa.'",
		"'I've e-mailed only a few kindred souls I think I can trust, not telling them anything about my human life, naturally. There are great advantages to my continued silence.'",
		"'No on knows, period. I live in fear that my intolerant neighbours, colleagues, or relatives will turn me over to the Thought Police! I post only under condition of anonymity. I'm going out on a limb even assembling my Dragon Code!'",
		"Variable"},
		{"Dragon-Friend: ",
		"'Have you noticed? you've started growing scales!'",
		"'Popular with dragons - you exchange overnight visits.'",
		"'Reasonably popular - you exchange social visits.'",
		"'Polite acquaintance - you exchange social pleasantries.'",
		"'Tolerance - they don't eat you, you don't try to slice them!'",
		"'Irritant - they think about having you over for lunch!'",
		"'Maddening - they think about a quick snack ... now!'",
		"'Infuriating - you're not good enough for a snack.'",
		"'Cold fury - they're going to hunt you, and find you, and...'"}
	};
	unsigned int i;
	char ctemp[100];
	strcat(out,info[misc][0]);
	// Is there a quoted property?
	strcpy(ctemp,tag);
	for (i=0;i<strlen(tag);i++)
	{
		if (tag[i]=='\"')
		{
			strcpy(ctemp,tag);
			ctemp[i]=0;
			break;
		}
	}
	if (SimpleTag(ctemp,out,info[misc][1],info[misc][2],info[misc][3],
		info[misc][4],info[misc][5],info[misc][6],info[misc][7],
		info[misc][8],info[misc][9],info[misc][10])==0)
	{
		strcat(out,"<Unrecognized modifier>");
	}
	if (strlen(ctemp)!=strlen(tag))		// Found a quote
	{
		strcat(out,"  ");
		strcat(out,&tag[strlen(ctemp)]);
	}
}

int DC2GHandlerSub(char tag[], char *out)
{
	DictionaryEntry gt[]={
		{"f","Female"},
		{"h","Hermaphrodite"},
		{"m","Male"},
		{"n","Neuter"},
		{"p","Pseudo-hermaphrodite"},
		{"~","Variable"},
		{"?","Unknown"},
		{"",""},{"",""}};
	int ret;

	ret=Dictionary(gt,tag,0);
	if (strlen(gt[ret].key))
	{
		strcat(out,gt[ret].out);
	} else {
		strcat(out,"<Unrecognized modifier>");
	}
	return strlen(gt[ret].key);
}

void DC2GHandler(char tag[], char *out, int misc)
{
	unsigned int pos,outlen;
	strcat(out,"Gender: ");
	outlen=strlen(out);
	if (!strlen(tag))
	{
		strcat(out,"None");
		return;
	}
	pos=DC2GHandlerSub(tag,out);
	if (pos<strlen(tag))
	{
		if (tag[pos]=='>')		// Changing into
		{
			strcat(out," in transition to ");
			DC2GHandlerSub(&tag[pos+1],out);
		} else if (tag[pos]=='/') {		// In another body
			out[outlen]=0;
			DC2GHandlerSub(&tag[pos+1],out);
			strcat(out," in a ");
			DC2GHandlerSub(tag,out);
			strcat(out," body");
		} else {
			strcat(out,"<Unrecognized modifier>");
		}
	}
}

void DC2LHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	float curnum;
	char units[20],ctemp[50];
	strcat(out,"Length: ");
	pos=SimpleTag(tag,out,"Celestial - 'I hate it when planets get in my mouth when I yawn!'",
		"Mistaken for mountain ranges - 'Sorry Mr Battleship, I didn't see you!'",
		"Can't see own tail on a foggy day - 'Can you say \"jungle gym\"?'",
		"Godzilla-sized - 'They look up to me. Literally.'",
		"Draco-sized - 'About as normal as dragons get.'",
		"Human-sized - 'Please, don't step on the tail...'",
		"Dog-sized - 'Please, don't step on me...'",
		"Pocket Dragon-sized or below - 'Please, don't sneeze...'",
		"Microscopic - 'Honey, I shrunk the dragon!'",
		"Variable - 'I change size on a whim!'");
	if (pos==0&&strcmp(tag,"^")==0)
	{
		strcat(out,"One-Dragon-Sized - \'I am just long enough to reach the ground!\'");
		return;
	}
	if (pos<strlen(tag))
	{
		// Quantitative
		if (pos)
			strcat(out,"; ");
		strcpy(units," feet");
		while (pos<strlen(tag))
		{
			curnum=(float)atof(&tag[pos]);
			while(!isalpha(tag[pos])&&pos<strlen(tag)) pos++;
			switch (tag[pos])
			{
			case 'i':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," inch(es) ");
				strcat(out,units);
				strcat(out,"long ");
				break;
			case 'f':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," foot ");
				strcat(out,units);
				strcat(out,"long ");
				break;
			case 'y':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," yard(s) ");
				strcat(out,units);
				strcat(out,"long ");
				break;
			case 'c':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," centimeter(s) ");
				strcat(out,units);
				strcat(out,"long ");
				break;
			case 'm':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," meter(s) ");
				strcat(out,units);
				strcat(out,"long ");
				break;
			case 'k':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," kilometer(s) ");
				strcat(out,units);
				strcat(out,"long ");
				break;
			case 'a':
				sprintf(ctemp,"%g%sarms ",curnum,units);
				strcat(out,ctemp);
				break;
			case 'l':
				sprintf(ctemp,"%g%slegs ",curnum,units);
				strcat(out,ctemp);
				break;
			case 'n':
				sprintf(ctemp,"%g%sneck ",curnum,units);
				strcat(out,ctemp);
				break;
			case 't':
				sprintf(ctemp,"%g%stail ",curnum,units);
				strcat(out,ctemp);
				break;
			case 'w':
				sprintf(ctemp,"%g%swingspan ",curnum,units);
				strcat(out,ctemp);
				break;
			default:
				strcat(out,"<Unrecognized modifier>");
			}
			pos++;
		}
	}
}

void DC2THandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	float curnum;
	char units[20];
	strcat(out,"Weight: ");
	pos=SimpleTag(tag,out,"Black-Hole - 'Everything gravitates towards me eventually!'",
		"Massive - 'I've been eating rocks again!'",
		"Obese - 'I'm on a sea-food diet ... I see food and eat it!'",
		"Over-weight - 'I've been eating too many doughnuts again!'",
		"Normal - 'I'm as normal a weight as one gets.'",
		"Under-weight - 'I've not had a good meal for ages!'",
		"Buoyant - 'I've been breathing helium again!'",
		"Feather-weight - 'I get blown about by the wind.'",
		"Weightless - 'I'm made out of gossamer.'",
		"Variable");
	if (pos<strlen(tag))
	{
		// Quantitative
		if (pos)
			strcat(out,"; ");
		while (pos<strlen(tag))
		{
			curnum=(float)atof(&tag[pos]);
			while(!isalpha(tag[pos])&&tag[pos]!='\"'&&pos<strlen(tag)) pos++;
			switch (tag[pos])
			{
			case 'c':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," hundredweight ");
				strcat(out,units);
				break;
			case 'g':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," gram(s) ");
				strcat(out,units);
				break;
			case 'k':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," kilogram(s) ");
				strcat(out,units);
				strcat(out,"long ");
				break;
			case 'l':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," pound(s) ");
				strcat(out,units);
				break;
			case 'o':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," ounce(s) ");
				strcat(out,units);
				break;
			case 's':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," stone(s) ");
				strcat(out,units);
				break;
			case 't':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				strcpy(units," ton(s) ");
				strcat(out,units);
				break;
			case '\"':
				sprintf(units,"%g",curnum);
				strcat(out,units);
				sprintf(units," %s ",&tag[pos]);
				strcat(out,units);
				pos+=strlen(&tag[pos]); 
				break;
			default:
				strcat(out,"<Unrecognized modifier>");
			}
			pos++;
		}
	}
}

void DC2PHandler(char tag[], char *out, int misc)
{
	// Grr.  Evil plurals
	DictionaryEntry pt[]={
		{"a^","Webbed arms"},
		{"a+","Extra arm"},
		{"a-","One arm"},
		{"a!","Many arms"},
		{"a~","Variable number of arms"},
		{"a","A pair of arms"},
		{"a","arm(s)"},

		{"f^","Webbed fore-limbs"},
		{"f+","Extra fore-limb"},
		{"f-","One fore-limb"},
		{"f!","Many fore-limbs"},
		{"f~","Variable number of fore-limbs"},
		{"f","A pair of fore-limbs"},
		{"f","fore-limb(s)"},

		{"h^","Webbed head"},
		{"h+","Extra head"},
		{"h-","No head"},
		{"h!","Many heads"},
		{"h~","Variable number of heads"},
		{"h","A head"},
		{"h","head(s)"},

		{"k^","Webbed crest"},
		{"k+","Extra crest"},
		{"k-","One crest"},
		{"k!","Many crests"},
		{"k~","Variable number of crests"},
		{"k","A crest"},
		{"k","crest(s)"},

		{"l^","Webbed legs"},
		{"l+","Extra leg"},
		{"l-","One leg"},
		{"l!","Many legs"},
		{"l~","Variable number of legs"},
		{"l","A pair of legs"},
		{"l","leg(s)"},
		
		{"p^","Webbed paddles/flukes/fins"},
		{"p+","Extra paddle/fluke/fin"},
		{"p-","One paddle/fluke/fin"},
		{"p!","Many paddles/flukes/fins"},
		{"p~","Variable number of paddles/flukes/fins"},
		{"p","A pair of paddles/flukes/fins"},
		{"p","paddle(s)/fluke(s)/fin(s)"},

		{"t^","Webbed tail"},
		{"t+","Extra tail"},
		{"t-","No tail"},
		{"t!","Many tails"},
		{"t~","Variable number of tails"},
		{"t","A tail"},
		{"t","tail(s)"},

		{"v^","Webbed horns"},	// Whatever
		{"v+","Extra horn"},
		{"v-","No horns"},
		{"v!","Many horns"},
		{"v~","Variable number of tails"},
		{"v","Horns"},
		{"v","horn(s)"},

		{"w`^","Webbed multi-purpose wings"},
		{"w`+","Extra multi-purpose wings"},
		{"w`-","One multi-purpose wing"},
		{"w`!","Many multi-purpose wings"},
		{"w`~","Variable number of multi-purpose wings"},
		{"w`","A pair of multi-purpose wings"},
		{"w`","multi-purpose wing(s)"},

		{"w'^","Webbed multi-purpose wings"},
		{"w'+","Extra multi-purpose wings"},
		{"w'-","One multi-purpose wing"},
		{"w'!","Many multi-purpose wings"},
		{"w'~","Variable number of multi-purpose wings"},
		{"w'","A pair of multi-purpose wings"},
		{"w'","multi-purpose wing(s)"},

		{"w^","Webbed wings"},		// Umm...
		{"w+","Extra wing"},
		{"w-","One wing"},
		{"w!","Many wings"},
		{"w~","Variable number of wings"},
		{"w","A pair of wings"},
		{"w","wing(s)"},
		{"",""},
		{"",""}};
	unsigned int pos;
	int ret;
	char ctemp[20];
	pos=0;
	strcat(out,"Appendages: ");
	while(pos<strlen(tag))
	{
		ret=Dictionary(pt,&tag[pos],0);
		if (strlen(pt[ret].key))
		{
			// Is there a number modifier?
			if (atoi(&tag[pos+strlen(pt[ret].key)])!=0)
			{
				sprintf(ctemp,"%d ",atoi(&tag[pos+strlen(pt[ret].key)]));
				strcat(out,ctemp);
				ret++;
				strcat(out,pt[ret].out);
				strcat(out,"  ");
				pos++;
				while(pos<strlen(tag)&&!isalpha(tag[pos])) pos++;
			} else {
				strcat(out,pt[ret].out);
				strcat(out,"  ");
				pos+=strlen(pt[ret].key);
			}
		} else {
			strcat(out,"<Unrecognized modifier>");
			pos++;
			while(pos<strlen(tag)&&!isalpha(tag[pos])) pos++;
		}
	}
}

void DC2SkHandler(char tag[], char *out, int misc)
{
	DictionaryEntry skt[]={
		{"b","Bark"},
		{"c","Cellulose"},
		{"e","Exoskeleton"},
		{"f","Feathers"},
		{"h","Hide"},
		{"k","Skin"},
		{"l","Leather"},
		{"m","Metal"},
		{"r","Rock"},
		{"s","Scales"},
		{"u","Fur"},
		{"x","Crystals"},
		{"",""},
		{"",""}};
	DictionaryEntry skt2[]={
		{",a","arms"},
		{",b","belly"},
		{",h","head"},
		{",l","legs"},
		{",n","neck"},
		{",t","tail"},
		{",w","wings"},
		{"",""},
		{"",""}};
	unsigned int pos;
	int ret;
	char part[20];
	strcat(out,"Skin-Type: ");
	pos=0;
	strcpy(part,"");
	if (!strlen(tag))
	{
		strcat(out,"None");
	}
	while(pos<strlen(tag))
	{
		ret=Dictionary(skt,&tag[pos],0);
		if (strlen(skt[ret].key))
		{
			strcat(out,skt[ret].out);
			if (strlen(part))
			{
				strcat(out," on ");
				strcat(out,part);
			}
			strcat(out,"  ");
			pos+=strlen(skt[ret].key);
		} else {
			ret=Dictionary(skt2,&tag[pos],0);
			if (strlen(skt2[ret].key))
			{
				strcpy(part,skt2[ret].out);
				pos+=strlen(skt2[ret].key);
			} else {
				//strcat(out,"<Unrecognized modifier>");
				strcat(out,"None");
				if (strlen(part))
				{
					strcat(out," on ");
					strcat(out,part);
				}
				strcat(out,"  ");
				pos++;
			}
		}
	}
}

int DC2CHandlerSingle(char tag[], char *out)
{
	DictionaryEntry ct[]={
		{"ag","silver"},
		{"am","amber"},
		{"aq","aquamarine"},
		{"au","gold"},
		{"bk","black"},
		{"bl","blue"},
		{"br","brown"},
		{"bs","brass"},
		{"bz","bronze"},
		{"ch","chromium"},
		{"cu","copper"},
		{"cy","cyan"},
		{"eb","ebony"},
		{"fs","flesh"},
		{"gr","green"},
		{"gy","grey"},
		{"hg","mercury/quicksilver"},
		{"in","indigo"},
		{"iv","ivory"},
		{"ja","jade"},
		{"ma","magenta"},
		{"mv","mauve"},
		{"or","orange"},
		{"pi","pink"},
		{"pu","purple"},
		{"rb","rainbow"},
		{"re","red"},
		{"ta","tan"},
		{"tu","turquoise"},
		{"mb","umber"},
		{"vi","violet"},
		{"wh","white"},
		{"ye","yellow"},
		{"~","chameleonic"},
		{"?","unknown"},
		{"",""},
		{"",""}};
	int ret;
	unsigned int pos;
	int lightcount,darkcount,metallic,transparent,luminescent,pearlescent,glittery;
	pos=0;
	if (!strlen(tag))
	{
		strcpy(out,"Colorless");
		pos++;
	} else {
		ret=Dictionary(ct,tag,0);
		if (strlen(ct[ret].out)==0)
		{
			strcpy(out,"<Unrecognized modifier>");
			pos++;
		} else {
			pos+=strlen(ct[ret].key);
			lightcount=darkcount=metallic=transparent=luminescent=pearlescent=glittery=0;
			while (tag[pos]=='+'||tag[pos]=='-'||tag[pos]=='^'||tag[pos]=='_'||tag[pos]=='\'')
			{
				switch(tag[pos])
				{
				case '+':
					lightcount++;
					break;
				case '-':
					darkcount++;
					break;
				case '^':
					metallic++;
					break;
				case '_':
					transparent++;
					break;
				case '\'':
					luminescent++;
					break;
				case '%':
					pearlescent++;	// Bah!  Everything is Perl these days
					break;
				case '!':
					glittery++;
					break;
				}
				pos++;
			}
			strcpy(out,"");
			while(lightcount>1) { strcat(out,"very "); lightcount--; }
			if (lightcount) strcat(out,"light ");
			while(darkcount>1) { strcat(out,"very "); darkcount--; }
			if (darkcount) strcat(out,"dark ");
			while(metallic>1) { strcat(out,"very "); metallic--; }
			if (metallic) strcat(out,"metallic ");
			while(transparent>1) { strcat(out,"very "); transparent--; }
			if (transparent) strcat(out,"transparent ");
			while(luminescent>1) { strcat(out,"very "); luminescent--; }
			if (luminescent) strcat(out,"luminescent ");
			while(pearlescent>1) { strcat(out,"very "); pearlescent--; }
			if (pearlescent) strcat(out,"pearlescent ");
			while(glittery>1) { strcat(out,"very "); glittery--; }
			if (glittery) strcat(out,"glittery ");
			strcat(out,ct[ret].out);
			out[0]=toupper(out[0]);
		}
	}
	return pos;
}

int DC2CHandlerSub(char tag[], char *out, char plaid=0, char patterned=0)
{
	char ctemp[100];
	unsigned int pos;
	pos=DC2CHandlerSingle(tag,ctemp);
	strcat(out,ctemp);
	if (pos<strlen(tag))
	{
		switch (tag[pos])
		{
		case '|':
			strcat(out," with ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			strcat(out," stripes");
			break;
		case '=':
			strcat(out," with ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			strcat(out," bands");
			break;
		case ':':
			strcat(out," with ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			strcat(out," spots");
			break;
		case '*':
			strcat(out," with ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			strcat(out," stars");
			break;
		case '@':
			strcat(out," mottled with ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			break;
		case '\\':
			strcat(out," with ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			strcat(out," irredescence");
			break;
		case '/':
			strcat(out," and ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			break;
		case '#':
			if (!plaid)
			{
				strcat(out," plaid with ");
			} else {
				strcat(out," and ");
			}
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out,1);
			break;
		case '&':
			if (!patterned)
			{
				if (tag[pos+1]!='1')
				{
					strcat(out," patterned with ");
				} else {
					strcat(out," marble with ");
					pos++;
				}
			} else {
				strcat(out," and ");
			}
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out,plaid,1);
			break;
		case '>':
			strcat(out," changing to ");
			pos++;
			pos+=DC2CHandlerSub(&tag[pos],out);
			break;
		case ',':
			return pos;
		default:
			strcat(out,"<Unrecognized modifier>");
			pos++;
		}
	}
	return pos;
}

void DC2CHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ct[]={
		{",a","arms"},
		{",b","belly"},
		{",c","claws"},
		{",e","eyes"},
		{",f","fur"},
		{",h","head"},
		{",k","crest"},
		{",l","legs"},
		{",n","neck"},
		{",p","points"},
		{",s","spines"},
		{",t","tail"},
		{",u","aura"},
		{",v","horns"},
		{",w","wings"},
		{"",""},
		{"",""}};
	unsigned int pos;
	int ret;
	strcat(out,"Color: ");
	pos=0;
	pos=DC2CHandlerSub(tag,out);
	while(pos<strlen(tag))
	{
		strcat(out,"  ");
		ret=Dictionary(ct,&tag[pos],0);
		if (strlen(ct[ret].key))
		{
			pos+=strlen(ct[ret].key);
			pos+=DC2CHandlerSub(&tag[pos],out);
			strcat(out," ");
			strcat(out,ct[ret].out);
		} else {
			strcat(out,"<Unrecognized modifier>");
			pos++;
		}
	}
}

int DC2BHandlerSingle(char tag[], char *out)
{
	DictionaryEntry bt[]={
		{"ac","acid"},
		{"co","cold/frost"},
		{"en","enchantment"},
		{"eg","energy"},
		{"fl","flame/fire"},
		{"he","heat"},
		{"ic","ice"},
		{"la","lava/magma"},
		{"ph","photons/light"},
		{"pl","plasma"},
		{"ro","rot"},
		{"sm","smoke"},
		{"st","steam"},
		{"su","sulphur"},
		{"vg","volcanic gases"},
		{"wa","water"},
		{"wi","wind"},
		{"zz","electricity/lightning"},
		{"-","no breath-weapon"},
		{"?","unknown"},
		{"~","anything"},
		{"",""},
		{"",""}};
	unsigned int pos;
	int ret;
	char ctemp[256];

	ret=Dictionary(bt,tag,0);
	if (strlen(bt[ret].key))
	{
		pos=strlen(bt[ret].key);
		strcpy(out,bt[ret].out);
		while (pos<strlen(tag))
		{
			switch(tag[pos])
			{
			case '|':
				strcat(out," beam");
				pos++;
				break;
			case '#':
				strcat(out," cloud");
				pos++;
				break;
			case '\"':
				strcpy(ctemp,out);
				ret=1;
				while(ret+pos<strlen(tag)&&tag[ret+pos]!='\"') ret++;
				ret++;
				out[ret]=0;
				strncpy(out,&tag[pos],ret);
				strcat(out," ");
				strcat(out,ctemp);
				pos+=ret;
				break;
			case '/':
				strcat(out," and ");
				pos++;
				pos+=DC2BHandlerSingle(&tag[pos],ctemp);
				strcat(out,ctemp);
				break;
			default:
				strcat(out,"<Unrecognized modifier>");
				pos++;
				break;
			}
		}
	} else if (strlen(tag)) {
		strcpy(out,"<Unrecognized modifier>");
	}
	return pos;
}

void DC2BHandler(char tag[], char *out, int misc)
{
	char ctemp[256];
	strcat(out,"Breath-weapon: ");
	DC2BHandlerSingle(tag,ctemp);
	ctemp[0]=toupper(ctemp[0]);
	strcat(out,ctemp);
}

int DC2AHandlerSub(char tag[], char *out)
{
	DictionaryEntry at[]={
		{"+++!","Ancient"},
		{"+++","Venerable"},
		{"++","Old enough to know better"},
		{"+","Been around"},
		{"---!","An egg"},
		{"---","Hatchling"},
		{"--","Still under Mom's (or Dad's) wing"},
		{"-","Young adult"},
		{"~","Variable"},
		{"?","Unknown"},
		{"",""},
		{"",""}};
	int ret;

	ret=Dictionary(at,tag,0);
	if (strlen(at[ret].key))
	{
		strcat(out,at[ret].out);
		return strlen(at[ret].key);
	} else {
		if (tag[0]=='r'||tag[0]=='v'||tag[0]=='('||tag[0]==')')
			strcat(out,"Mature adult");
		else
			strcat(out,"<Unrecognized modifier>");
		return 1;
	}
}

void DC2AHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	pos=0;
	strcat(out,"Age: ");
	if (!strlen(tag))
	{
		strcat(out,"Mature adult");
	}
	while(pos<strlen(tag))
	{
		switch(tag[pos])
		{
		case 'r':
			pos++;
			pos+=DC2AHandlerSub(&tag[pos],out);
			strcat(out," in real life  ");
			break;
		case 'v':
			pos++;
			pos+=DC2AHandlerSub(&tag[pos],out);
			strcat(out," on the net  ");
			break;
		case '(':		// Just ignore paranthesis
		case ')':
			pos++;
			break;
		default:
			pos+=DC2AHandlerSub(&tag[pos],out);
			strcat(out,"  ");
		}
	}
}

void DC2FrHandler(char tag[], char *out, int misc)
{
	unsigned int pos,postemp;
	strcat(out,"Fruitiness: ");
	if (!strlen(tag))
	{
		strcat(out,"'Old fruits know my name and what I'm like.'/'Squeeze me! I'm ripe!'");
	}
	pos=0;
	while(pos<strlen(tag))
	{
		switch(tag[pos])
		{
		case '^':
			strcat(out,"Have joined, left, returned, etc.  ");
			pos++;
			break;
		case '*':
			strcat(out,"I got flamed off the newsgroup for yelling at Shim! :8)  ");
			pos++;
			break;
		case '\"':
			strcat(out,&tag[pos]);
			strcat(out,"  ");
			pos=strlen(tag);
			break;
		default:
			postemp=pos;
			pos+=SimpleTagRel(&tag[pos],out,
				"'I remember when humans ruled the newsgroup!'/'I'm too fruity for my hat!'",
				"'I remember Dalvenjah posting regularly.'/'I am well into the fermentation process.'",
				"'I've been here for a couple of years or so.'/'I have accidentally been used in food fights.'",
				"'I've been here just over a year now.'/'Friends have spotted me in fruit salads.'",
				"'Old fruits know my name and what I'm like.'/'Squeeze me! I'm ripe!'",
				"'My name is known, but that's about it.'/'I haven't fallen off the tree yet!'",
				"'They're still responding to my welcome post.'/'It can be said that I am very a-peel-ing!'",
				"'I haven't posted yet, I'm just lurking!'/'A squirrel tried to hoard me the other day.'",
				"'I've never heard of AFD!'/'I'm only a sparkle in the eye of the tree!'",
				"Variable");
			if (postemp==pos)		// So it can't crash
			{
				strcat(out,"<Unrecognized modifier>");
				pos++;
			}
			strcat(out,"  ");
			break;
		}
	}
}

void DC2NHandler(char tag[], char *out, int misc)
{
	DictionaryEntry nt[]={
		{"a","Air - 'I live in the clouds!'"},
		{"e","Earth - 'I prefer to dwell underground!'"},
		{"f","Fire - 'I am a child of the flame!'"},
		{"i","Ice - 'I thrive in the cold polar winds and the tundra is my playground!'"},
		{"j","Jungle - 'I only feel happy in the tropical rainforests of my home.'"},
		{"m","Mountains - 'I am only truly happy when surrounded by tall peaks!'"},
		{"n","Nature - 'I was born in the wilderness, and that's where I'm happiest!'"},
		{"p","Plains - 'I prefer to walk on a carpet of green!'"},
		{"r","Rock - 'I have a bed of marble and a granite pillow!'"},
		{"s","Space - 'The dark lifeless void is my home!'"},
		{"t","Trees - 'I like swinging through the branches!'"},
		{"u","Urban - 'Cities and towns hold fascination for me!'"},
		{"w","Water - 'I am only truly at home in water!'"},
		{"^","Ethereal - 'I exist in the folds of reality!'"},
		{"!","Imaginary - 'I am a figment of my imagination!'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Native-Land: ");
	if (!strlen(tag))
		return;
	ret=Dictionary(nt,tag,1);
	if (strlen(nt[ret].key))
	{
		strcat(out,nt[ret].out);
	} else {
		strcat(out,"<Unrecognized modifier>");
	}
}

int DC2MHandlerSub(char tag[], char *out)
{
	DictionaryEntry at[]={
		{"+++!","'I'm mated in real life to my on-line mate's real life self as well'"},
		{"+++","'Am I mated? Ask my hatchlings'"},
		{"++","'Ask my significant other'"},
		{"+","'Ask my mate-to-be'"},
		{"---!","'Not only am I single, but I despise the idea of mating'"},
		{"---","'I'm single and very proud of it'"},
		{"--","'Ask my (poker/football/bridge/Trivial Pursuit) buddies'"},
		{"-","Don't ask"},
		{"/","'Ask me, and I'll ask your snout to meet your kidneys'"},
		{"",""},
		{"",""}};
	int ret;
	unsigned int pos;
	char ctemp[2];

	ret=Dictionary(at,tag,0);
	if (strlen(at[ret].key))
	{
		strcat(out,at[ret].out);
		pos=strlen(at[ret].key);
		while(pos<strlen(tag))
		{
			if (atoi(&tag[pos])!=0)
			{
				strcat(out," (");
				ctemp[1]=0;
				while(isdigit(tag[pos]))
				{
					ctemp[0]=tag[pos];
					strcat(out,ctemp);
					pos++;
				}
				strcat(out," mates)");
			} else if (tag[pos]=='|') {
				strcat(out," (separated from ");
				ctemp[1]=0;
				pos++;
				while(isdigit(tag[pos]))
				{
					ctemp[0]=tag[pos];
					strcat(out,ctemp);
					pos++;
				}
				strcat(out,")");
			} else if (tag[pos]=='*') {
				strcat(out," (separated from ");
				ctemp[1]=0;
				pos++;
				while(isdigit(tag[pos]))
				{
					ctemp[0]=tag[pos];
					strcat(out,ctemp);
					pos++;
				}
				strcat(out," bitterly)");
			} else if (tag[pos]=='_') {
				strcat(out," (");
				ctemp[1]=0;
				pos++;
				while(isdigit(tag[pos]))
				{
					ctemp[0]=tag[pos];
					strcat(out,ctemp);
					pos++;
				}
				strcat(out," dead)");
			} else if (tag[pos]=='x'||tag[pos]=='r'||tag[pos]=='v'||tag[pos]=='('||tag[pos]==')'||tag[pos]==' ') {
				return pos;
			} else {
				strcat(out,"<Unrecognized modifier>");
				pos++;
			}
		}
		return pos;
	} else {
//		if (tag[0]=='r'||tag[0]=='v'||tag[0]=='('||tag[0]==')')
			strcat(out,"'Ask me, and I might just say yes'");
//		else
//			strcat(out,"<Unrecognized modifier>");
		return 1;
	}
}

void DC2MHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	int distance;
	pos=0;
	strcat(out,"Mating-Status: ");
	if (!strlen(tag))
	{
		strcat(out,"'Ask me, and I might just say yes'");
		return;
	}
	distance=0;
	while (pos<strlen(tag))
	{
		switch(tag[pos])
		{
		case ' ':
			distance++;
			pos++;
			break;
		case 'x':
			if (tag[pos+1]=='x')
			{
				strcat(out,"Card-carrying member of the XX Conspiracy  ");
				pos+=2;
			} else if (tag[pos+1]=='y') {
				strcat(out,"Card-carrying member of the XY Conspiracy  ");
				pos+=2;
			} else {
				strcat(out,"<Unrecognized modifier>");
				pos++;
			}
			break;
		case 'r':
			pos++;
			while(pos<strlen(tag)&&tag[pos++]==' ') distance++;
			pos+=DC2MHandlerSub(&tag[pos],out);
			if (distance)
				strcat(out," (distant)");
			strcat(out," in real life");
			strcat(out,"  ");
			break;
		case 'v':
			pos++;
			while(pos<strlen(tag)&&tag[pos++]==' ') distance++;
			pos+=DC2MHandlerSub(&tag[pos],out);
			if (distance)
				strcat(out," (distant)");
			strcat(out," on the net");
			strcat(out,"  ");
			break;
		case '(':	// Ignore
		case ')':
			pos++;
			break;
		default:
			pos+=DC2MHandlerSub(&tag[pos],out);
			if (distance)
				strcat(out," (distant)");
			strcat(out,"  ");
			break;
		}
	}
}

int DC2OHandlerSub(char tag[], char *out)
{
	DictionaryEntry ot[]={
		{"+++!","'I have many of them, and none of them look like leaving yet'"},
		{"+++","'I have several that haven't fledged yet'"},
		{"++","'I have a couple that have not yet left home'"},
		{"+","'I've got one still in the nest'"},
		{"---!","'I've had many, but they've all flown away now'"},
		{"---","'I've had several, all successfully fledged'"},
		{"--","'I had a couple, but they're living their own lives now'"},
		{"-","'I had one, but they've left home'"},
		{"?","'I lost track long ago. If it meeps, I feed it'"},
		{"~","'I have a variable number, depending on who I'm fostering this month'"},
		{"/","'If I ever heard one was my fault, I'd faint'"},
		{"",""},
		{"",""}};
	int ret;
	unsigned int pos;
	char ctemp[2];
	int adopted;

	adopted=0;
	if (tag[0]=='a')	// Adopted modifier
	{
		adopted=1;
	}
	ret=Dictionary(ot,&tag[adopted],0);
	if (strlen(ot[ret].key))
	{
		// Is it a numeric?
		if (isdigit(tag[1+adopted]))
		{
			pos=1+adopted;
			if (tag[adopted]=='+')
			{
				ctemp[1]=0;
				while(pos<strlen(tag)&&isdigit(tag[pos]))
				{
					ctemp[0]=tag[pos];
					strcat(out,ctemp);
					pos++;
				}
				strcat(out," haven't left home yet");
				if (adopted) strcat(out," (adopted)");
				return pos;
			} else if (tag[adopted]=='-') {
				ctemp[1]=0;
				while(pos<strlen(tag)&&isdigit(tag[pos]))
				{
					ctemp[0]=tag[pos];
					strcat(out,ctemp);
					pos++;
				}
				strcat(out," have flown the coop");
				if (adopted) strcat(out," (adopted)");
				return pos;
			} else {
				strcat(out,"<Unrecognized modifier>");
				return 1;
			}
		} else {
			strcat(out,ot[ret].out);
		}
		if (adopted) strcat(out," (adopted)");
		return strlen(ot[ret].key)+adopted;
	} else {
		strcat(out,"'There's just me (and my mate, if applicable)'");
		if (adopted) strcat(out," (adopted)");
		return 1+adopted;
	}
}

void DC2OHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	strcat(out,"Offspring: ");
	pos=0;
	if (!strlen(tag))
	{
		strcat(out,"'There's just me (and my mate, if applicable)'");
	}
	while (pos<strlen(tag))
	{
		switch (tag[pos])
		{
		case 'r':
			pos++;
			pos+=DC2OHandlerSub(&tag[pos],out);
			strcat(out," in real life.  ");
			break;
		case 'v':
			pos++;
			pos+=DC2OHandlerSub(&tag[pos],out);
			strcat(out," on the net.  ");
			break;
		case '(':		// Ignore
		case ')':
			pos++;
			break;
		default:
			pos+=DC2OHandlerSub(tag,out);
			strcat(out,".  ");
			break;
		}
	}
}

int DC2FHandlerSub(char tag[], char *out)
{
	DictionaryEntry ft[]={
		{"c","carniverous"},
		{"f","fluids"},
		{"j","junk food"},
		{"m","mineral-eater"},
		{"o","omnivorous"},
		{"p","photosynthesiser"},
		{"v","vegetarian"},
		{"",""},
		{"",""}};
	int group,ret;
	unsigned int pos;

	// Is the group on the beginning?
	if (isalpha(tag[0]))
	{
		group=Dictionary(ft,tag,0);
		pos=strlen(ft[group].key);
	} else {		// Is it on the end
		group=-1;
		pos=1;
		while(pos<strlen(tag)&&!isalpha(tag[pos])&&tag[pos]!='/') pos++;
		if (isalpha(tag[pos]))
		{
			group=Dictionary(ft,&tag[pos],0);
		} else {
			group=-1;
		}
		pos=0;
	}
	ret=SimpleTagRel(&tag[pos],out,
		"Insatiable - 'More! More! More!'",
		"Voracious - 'At least you don't have to wash up the plates now!'",
		"Glutton - 'I have three square meals an hour!'",
		"Overindulgent - 'One more of those won't hurt!'",
		"Normal for your species - 'I have three square meals a day!'",
		"Dieting - 'Only a soupon of that if you please!'",
		"Efficient - 'Only soup if you please!'",
		"Anorexic - 'Eating is something I do every blue moon!'",
		"Fasting - 'Duke Humphrey is an ideal dinner guest!'",
		"Anything  ");
	if (ret)
	{
		if (group!=-1)
		{
			if (strlen(ft[group].key))
			{
				strcat(out," (");
				strcat(out,ft[group].out);
				strcat(out,")");
				return ret+strlen(ft[group].key);
			} else {
				strcat(out," (<Unrecognized modifier>)");
				return 1;
			}
		}
		return pos+ret;
	} else {
		strcat(out,"<Unrecognized modifier>");
		return 1;
	}
}

void DC2FHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	strcat(out,"Diet: ");
	if (!strlen(tag))
	{
		strcat(out,"Normal for your species - 'I have three square meals a day!'");
		return;
	}
	pos=0;
	while(pos<strlen(tag))
	{
		pos+=DC2FHandlerSub(&tag[pos],out);
		if (tag[pos]=='/')
		{
			strcat(out," and ");
			pos++;
		}
	}
}

void DC2RHandler(char tag[], char *out, int misc)
{
	DictionaryEntry rt[]={
		{"+++!","'I AM as I describe myself, and no-one is going to persuade me otherwise. What you see is what your mind wants you to see!'"},
		{"+++","'I AM as I describe myself on-line. The human body is either an unfortunate mismatch, an illusion, or just my messenger.'"},
		{"++","'I tend to answer to my on-line name quicker than my given one. I strongly identify with this form. It would explain a lot if I wasn't really human, but I'm hedging my bets.'"},
		{"+","'It's a role, but it's a role that's part of my self-image, without which I would feel like something was missing.'"},
		{"---!","'I am role-playing. You are role-playing. If you try to correct me on this assumption, I will publicly flame you as insane.'"},
		{"---","'I consider all of this imaginary. You R+++ people scare me. Don't you think you're taking things a bit too seriously?'"},
		{"--","'This is a hobby for me. My character is just a product of my imagination, nothing more.'"},
		{"-","'It would be nice to be as I say I am on-line, but I've got some real doubts about whether such a thing is possible.'"},
		{"*","'Don't ask, don't tell.'"},
		{"?","'The thought just hasn't crossed my mind!'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Reality-Index: ");
	ret=Dictionary(rt,tag,1);
	if (strlen(rt[ret].key))
	{
		strcat(out,rt[ret].out);
	} else if (!strlen(tag)) {
		strcat(out,"'I haven't given much thought to whether I am more \"real\" in human form or in my on-line identity. I'm just me.'");
	} else {
		strcat(out,"<Unrecognized modifier>");
	}
}

void DC2AcHandler(char tag[], char *out, int misc)
{
	DictionaryEntry rt[]={
		{"+++!","'I have a T1 connection and am never off the 'net! I even order pizza across it!'"},
		{"+++","'This is my home! I spend as many waking moments as possible on-line.'"},
		{"++","'I'm on-line every day for about eight hours or so!'"},
		{"+","'I'm on-line for at least two hours every day!'"},
		{"---!","'If you see me, it must be raining frogs again!'"},
		{"---","'I'm lucky if I get on-line once a month!'"},
		{"--","'I can barely manage to connect once a week.'"},
		{"-","'I only get on occasionally during the week, but am here every weekend.'"},
		{"~","'I go on-line when I fee like it. It may be hours, it may be months!'"},
		{"?","'I really don't know when I'll be on-line again!'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Activity-Index: ");
	ret=Dictionary(rt,tag,1);
	if (strlen(rt[ret].key))
	{
		strcat(out,rt[ret].out);
	} else if (!strlen(tag)) {
		strcat(out,"'I get on every day, briefly, to check my mail and the newsgroups.'");
	} else {
		strcat(out,"<Unrecognized modifier>");
	}
}

int DC2JHandlerSub(char tag[], char *out)
{
	DictionaryEntry at[]={
		{"+++!","'Everything is a joke. I find something to laugh at everywhere I look ... and that includes you!'"},
		{"+++","'There's very little I won't laugh about ... and very little I won't do to make you laugh!'"},
		{"++","'Laughing is just so good for you! Even when you're being serious about something it's good to slip in the odd joke!'"},
		{"+","'I appreciate humour of all kinds, but I do know when to take things seriously.'"},
		{"---!","'Jokes, frivolity, they're all just works of the Devil!'"},
		{"---","'I can't think of the time when I last laughed.'"},
		{"--","'Some things are OK to laugh at, but many things are not. I'll take whatever you say seriously unless you put those cute little smileys by it!'"},
		{"-","'A joke a day keeps depression at bay ... but laughing out loud is bad for my health!'"},
		{"~","Variable"},
		{"?","Unknown"},
		{"",""},
		{"",""}};
	int ret;

	ret=Dictionary(at,tag,0);
	if (strlen(at[ret].key))
	{
		strcat(out,at[ret].out);
		return strlen(at[ret].key);
	} else {
		if (tag[0]=='r'||tag[0]=='v'||tag[0]=='('||tag[0]==')')
			strcat(out,"'I laugh at many things, but there's a time and a place for joking.'");
		else
			strcat(out,"<Unrecognized modifier>");
		return 1;
	}
}

void DC2JHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	pos=0;
	strcat(out,"Humour-Index: ");
	if (!strlen(tag))
	{
		strcat(out,"'I laugh at many things, but there's a time and a place for joking.'");
	}
	while(pos<strlen(tag))
	{
		switch(tag[pos])
		{
		case 'r':
			pos++;
			pos+=DC2JHandlerSub(&tag[pos],out);
			strcat(out," in real life  ");
			break;
		case 'v':
			pos++;
			pos+=DC2JHandlerSub(&tag[pos],out);
			strcat(out," on the net  ");
			break;
		case '(':		// Just ignore paranthesis
		case ')':
			pos++;
			break;
		default:
			pos+=DC2JHandlerSub(&tag[pos],out);
			strcat(out,"  ");
		}
	}
}

void DC2UHandler(char tag[], char *out, int misc)
{
	DictionaryEntry rt[]={
		{"+++!","'They call me 'The Vanisher'. Odd Beholder creatures that I only vaguely remember are still fighting holy wars over me. I've lost track of the number of planets my friends and I have personally shaped the history of. I'm not a god!'"},
		{"+++","'Not only do I remember most of my dozens (or hundreds) of past lives, but I remember a few things about you too!'"},
		{"++","'See that book over there? I wrote that in a different life!'"},
		{"+","'I can remember several past-life names, and am occasionally recognised too!'"},
		{"---!","'I blocked out my memories in self-defence after the incident with the Trout farm, the Spell of Growth, and the women's locker room at the Magic Academy! Total strangers occasionally give me odd looks and move to the other side of the street as I pass.'"},
		{"---","'Whatever it was I did, it made me end up here. I don't want to look!'"},
		{"--","'When I tried to do a past-life regression, I got a '404 Not found - Access denied'. Why don't past lives come with search engines?'"},
		{"-","'I'm pretty sure I've done something important, sometime, but I haven't got much of a clue as to what it was.'"},
		{"?","'What's a past life?'"},
		{"!","'This is my first life!'"},
		{"*","'I've been around a bit, I'm just not saying how many times!'"},
		{"2","'My name is Bono! :8)'"},
		{"~","Variable"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Ubiquity: ");
	ret=Dictionary(rt,tag,1);
	if (strlen(rt[ret].key))
	{
		strcat(out,rt[ret].out);
	} else if (!strlen(tag)) {
		strcat(out,"'I've done some soul-searching, and got real results! This isn't my first time around.'");
	} else {
		strcat(out,"<Unrecognized modifier>");
	}
}

void DC2IHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	pos=0;
	strcat(out,"Irritability: ");
	pos=SimpleTagRel(tag,out,
		"'You're just not going to catch me in a good mood! Live with it or you'd better like barbeque!'",
		"'I'd eat my mother if she looked at me wrong!'",
		"'Come too close and you're a cinder.'",
		"'Call me grumpy.'",
		"'I will defend my honour, but I will only attack with reason.'",
		"'Just don't call me lizard lips and you should be fine.'",
		"'I take everything in my stride, as opposed to my jaws!'",
		"'You could stab me with a ten-foot pike, and I wouldn't blink!'",
		"'There's nothing you can do that will make me lose my cool!'",
		"Variable");
	if (tag[pos]=='#')
		strcat(out,"  'But when I do get angry, I hope you like barbeque!'");
}

void DC2VHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	char ctemp[2];
	strcat(out,"Magical-Ability: ");
	pos=SimpleTagRel(tag,out,
		"'I have reached the pinnacle of my profession'",
		"'I'm reasonably adept in my field'",
		"'I know a number of spells'",
		"'I can perform a few cantrips with candles'",
		"'Magic is something I've never really looked into'",
		"'Magicians worry when I'm near'",
		"'Most magic seems to fail when I'm nearby'",
		"'Only a few spells seem to have an effect, but that could just be psychological'",
		"'Magic? What's that?'",
		"Variable");
	if (tag[pos]=='[')
	{
		strcat(out," of ");
		ctemp[1]=0;
		pos++;
		while(tag[pos]!=']'&&pos<strlen(tag))
		{
			ctemp[0]=tag[pos];
			strcat(out,ctemp);
			pos++;
		}
		strcat(out," magic");
	}
	strcat(out,".");
}

void DC2QHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	char ctemp[2];
	strcat(out,"Psy-Power: ");
	pos=SimpleTagRel(tag,out,
		"'There's almost nothing I can't do if I put my mind to it.'",
		"'I can move mountains, and not bit by bit!'",
		"'I know what you're thinking... did I use six psychic blasts or only five!!!'",
		"'I can talk to the odd spirit, or move very small rocks (churches, lead, ducks, etc.).'",
		"'I'm like a book, but haven't learned to read myself yet!'",
		"'Psychics have trouble communicating when I'm around.'",
		"'Only my very outer thoughts are exposed.'",
		"'Psionics just don't seem to have any affect.'",
		"'Not only am I immune to any Psionic effect, but I prevent them happening around me as well'",
		"Variable");
	if (tag[pos]=='[')
	{
		strcat(out," (");
		ctemp[1]=0;
		pos++;
		while(tag[pos]!=']'&&pos<strlen(tag))
		{
			ctemp[0]=tag[pos];
			strcat(out,ctemp);
			pos++;
		}
		strcat(out,")");
	}
}

void DC2TcHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	char ctemp[2];
	strcat(out,"Technology: ");
	pos=SimpleTagRel(tag,out,
		"'I write microcode in my spare time!'",
		"'I can program computers using assembly language.'",
		"'I can program computers using high-level languages.'",
		"'I can program the video.'",
		"'I haven't yet learned how to wire a plug!'",
		"'If a program has a bug, I'll find it!'",
		"'Electricity does funny things when I'm at the controls!'",
		"'Only the most basic mechanisms survive when I get hold of them!'",
		"'All items of technology fail when I'm near!'",
		"Variable");
	if (tag[pos]=='[')
	{
		strcat(out," (");
		ctemp[1]=0;
		pos++;
		while(tag[pos]!=']'&&pos<strlen(tag))
		{
			ctemp[0]=tag[pos];
			strcat(out,ctemp);
			pos++;
		}
		strcat(out,")");
	}
}

void DC2EHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	pos=0;
	strcat(out,"Huggability: ");
	pos=SimpleTagRel(tag,out,
		"'If it is living or dead, I'll hug it (to death)!'",
		"'If it is living, I'll hug it freely.'",
		"'I'm fairly free with my hugs, so try me!'",
		"'I'm selective, to a point, but give me a hug and I'll return it.'",
		"'I don't mind hugs from any of my friends, but I draw the line there.'",
		"'I'll accept hugs from my nearest and dearest, but no-one else.'",
		"'Hugging me is something that only my mate is allowed to do!'",
		"'Don't you dare hug me! I mean it!'",
		"'Don't even touch me ... in fact don't even think about touching me!'",
		"Variable");
	if (tag[pos]=='#')
		strcat(out,"  'But when you do, expect a violent reaction!'");
}

void DCFHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"+++","'I live exclusively off of town maidens'"},
		{"++","'I eat large game and the occasional human'"},
		{"+","'I eat mostly dairy products, like cows...'"},
		{"---","'I'm a vegetarian dragon. (You are?)'"},
		{"--","'I eat Caesar salads garnished with small furry animals'"},
		{"-","'I live off of small rodents and grubs'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Diet: ");
	if (!strlen(tag))
		strcat(out,"'I eat whatever I can catch'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCSHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"+++","'All my friends knew, but didn't believe me, so I ate them.'"},
		{"++","'All my friends know; so do my parents.'"},
		{"+","'A few of my closest friends know.'"},
		{"---","'No one knows, period.'"},
		{"--","'If anyone finds out, I eat them.'"},
		{"-","'One person knows, but they skipped town.'"},
		{"",""},
		{"",""}};
	int ret;
	unsigned int pos;

	pos=0;
	if (!strlen(tag))
		strcat(out,"'My closest friend knows.'");
	while (pos<strlen(tag))
	{
		switch (tag[pos])
		{
		case 'R':
			pos+=2;
			ret=Dictionary(ft,&tag[pos],0);
			if (strlen(ft[ret].key))
				strcat(out,ft[ret].out);
			else
				strcat(out,"'My closest friend knows.'");
			strcat(out," in real life  ");
			pos+=strlen(ft[ret].key);
			break;
		case 'V':
			pos+=2;
			ret=Dictionary(ft,&tag[pos],0);
			if (strlen(ft[ret].key))
				strcat(out,ft[ret].out);
			else
				strcat(out,"'My closest friend knows.'");
			strcat(out," on the net  ");
			pos+=strlen(ft[ret].key);
			break;
		case '(':	// Ignore
		case ')':
		case ' ':
			pos++;
			break;
		default:
			ret=Dictionary(ft,&tag[pos],0);
			if (strlen(ft[ret].key))
				strcat(out,ft[ret].out);
			else
				strcat(out,"'My closest friend knows.'");
			pos+=strlen(ft[ret].key);
			strcat(out,"  ");
		}
	}
}

void DCDfHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"++++","'Have you noticed?  You've started growing scales...'"},
		{"+++","'Popular with dragons; you exchange overnight visits'"},
		{"++","'Reasonably popular; you exchange social visits'"},
		{"+","'Polite acquaintance; you exchange social pleasantries'"},
		{"----","'Cold fury; they're gonna hunt you, and find you, and then...'"},
		{"---","'Infuriating; you're not good enough for a snack.'"},
		{"--","'Maddening; they think about a quick snack;  now!'"},
		{"-","'Irritant; they think about having you over for lunch'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Dragon Friend: ");
	if (!strlen(tag))
		strcat(out,"'Tolerance; they don't eat you, you don't try to slice them'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCHHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"++++","'Governments quake when they hear you're going to sell'"},
		{"+++","'You really *_can't_* tell when someone stole one item!'"},
		{"++","'You're comfortable;  but you wouldn't mind some more'"},
		{"+","'You're not down to your last copper, but ...'"},
		{"---","'No money, no lair;  at least you aren't starving'"},
		{"--","'The lair is rented, and cramped'"},
		{"-","'You and the bank own your lair'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Hoard size: ");
	if (!strlen(tag))
		strcat(out,"'You've got your own lair, unfurnished'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

int DCCHandlerSingle(char tag[], char *out)
{
	DictionaryEntry ct[]={
		{"R","Red"},
		{"O","Orange"},
		{"Y","Yellow"},
		{"G","Green"},
		{"J","Jade"},
		{"a","Aqua"},
		{"c","Cyan"},
		{"B","Blue"},
		{"b","Brown"},
		{"I","Indigo"},
		{"V","Violet"},
		{"P","Purple"},
		{"p","Pink"},
		{"m","Magenta"},
		{"t","Tan"},
		{"K","Black"},
		{"E","Ebon"},
		{"W","White"},
		{"g","Grey"},
		{"F","Flesh"},
		{"A","Gold"},
		{"S","Silver"},
		{"C","Copper"},
		{"Q","Mercury/quicksilver"},
		{"k","Chrome"},
		{"Z","Bronze"},
		{"Mb","Brass"},
		{"M","Metallic"},
		{"T","Transparent"},
		{"H","Luminous"},
		{"f","Flame/fire"},
		{"D","Dark"},
		{"L","Light"},
		{"-","Colorless"},
		{"",""},
		{"",""}};
	int ret;
	unsigned int pos;

	pos=0;

	while((isalpha(tag[pos])||tag[pos]=='-')&&tag[pos]!='w')
	{
		ret=Dictionary(ct,&tag[pos],0);
		if (strlen(ct[ret].key))
		{
			strcat(out,ct[ret].out);
			strcat(out," ");
			pos+=strlen(ct[ret].key);
		} else {
			strcat(out,"<Unrecognized modifier>");
			pos++;
		}
	}
	return pos;
}

void DCCHandler(char tag[], char *out, int misc)
{
	unsigned int pos;
	strcat(out,"Coloration: ");
	pos=0;
	while(pos<strlen(tag))
	{
		switch (tag[pos])
		{
		case '|':
			strcat(out,"with ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			strcat(out,"stripes ");
			break;
		case '.':
			strcat(out,"with ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			strcat(out,"spots ");
			break;
		case '@':
			strcat(out,"mottled with ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			break;
		case '^':
			strcat(out,"with a ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			strcat(out,"spine ");
			break;
		case 'w':
			strcat(out,"with ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			strcat(out,"wings ");
			break;
		case '~':
			strcat(out,"with a ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			strcat(out,"tail ");
			break;
		case '*':
			strcat(out,"with ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			strcat(out,"stars ");
			break;
		case ':':
			strcat(out,"with ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			strcat(out,"eyes ");
			break;
		case '#':
			strcat(out,"Plaid ");
			pos++;
			break;
		case '>':
			strcat(out,"changing to ");
			pos++;
			pos+=DCCHandlerSingle(&tag[pos],out);
			break;
		default:
			pos+=DCCHandlerSingle(&tag[pos], out);
			break;
		}
	}
}

void DCAHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"++++","Ancient"},
		{"+++","Venerable"},
		{"++","Old enough to know better"},
		{"+","You've been around"},
		{"----","An egg"},
		{"---","Hatchling"},
		{"--","Still under Mom's wing"},
		{"-","Young adult"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Age: ");
	if (!strlen(tag))
		strcat(out,"Mature adult");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCDollarHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"+++","'The coinage has my likeness on it. I own the whole kingdom.'"},
		{"++","'Besides humans, I kill other dragons for their wealth.'"},
		{"+","'Investments in human businesses. I do reposessions **CHOMP** personally.'"},
		{"---","'I'm a philanthropist. Gave it all to the poor. I think I'm a duck. QUACK!'"},
		{"--","'It was all stolen while I was out hunting hobbits!'"},
		{"-","'Bought up the weapons smiths. No more dragon-proof armor or vorpal swords!'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Money: ");
	if (!strlen(tag))
		strcat(out,"'Take your hands off my hoard or I'll take your hands.'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCMHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"+++","'Am I mated?  Ask my hatchlings!'"},
		{"++","'Ask my significant other!'"},
		{"+","'Ask my mate-to-be!  ]:=8)'"},
		{"---","'Here's my XX/XY Conspiracy Card.  Any questions?'"},
		{"--","'Ask my poker/football/bowling/etc. buddies!'"},
		{"-","'Don't ask! ]B=8P'"},
		{"+-+","Single with hatchlings"},
		{"(w)","'Widowed {:=8{'"},
		{"(m)","'Ask me, and I'll ask your snout to meet your kidneys.'"},
		{"**","'Ask my other half, who is at the keyboard with me! :-)'"},
		{" ++","Temporarily separated"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Mating status: ");
	if (!strlen(tag))
		strcat(out,"'Ask me, PLEASE! *hopefulsmiles*'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCM2Handler(char tag[], char *out, int misc)
{
	strcat(out,"Mating status: ");
	if (strlen(tag))
		strcat(out,"<Unrecognized modifier>");
	else
		strcat(out,"'Ask me, and I'll ask your snout to meet your kidneys.'");
}

void DCDHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"+++","'Wanna see my scales and fangs? :=8)'"},
		{"++","'I'm just waiting for my wings and tail to grow in!'"},
		{"+","'Ask my Inner Dragon.'"},
		{"---","'Now, I like dragons and all, but don't you think y'all are taking this a little too seriously?'"},
		{"--","'Certified greggil'"},
		{"-","'It is possible a distant relative was a dragon'"},
		{"(d)","'Don't ask, don't tell'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Draconity Index: ");
	if (!strlen(tag))
		strcat(out,"'I have draconic tendencies'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCD2Handler(char tag[], char *out, int misc)
{
	strcat(out,"Draconity Index: ");
	if (strlen(tag))
		strcat(out,"<Unrecognized modifier>");
	else
		strcat(out,"'Don't ask, don't tell'");
}

void DCWlHandler(char tag[], char *out, int misc)
{
	int wc,lc;
	unsigned int pos;
	char ctemp[100];
	strcat(out,"Wings/Legs: ");
	pos=wc=lc=0;
	for (pos=0;pos<strlen(tag);pos++)
	{
		switch (tag[pos])
		{
		case '+':
			wc++;
			break;
		case '*':
			lc++;
			break;
		default:
			strcat(out,"<Unrecognized modifier> ");
		}
	}
	sprintf(ctemp,"%d pair(s) of wings; %d pair(s) of legs",wc,lc);
	strcat(out,ctemp);
}

void DCFrHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"++++","'I remember when humans ruled the newsgroup'"},
		{"+++","'I remember Dalvenjah posting regularly'/'I am well into the fermentation process'"},
		{"++","'I've been here for well over a year'/'I have accidentally been used in food fights'"},
		{"+","'I've been here a year or so'/'Friends have spotted me in fruit salads'"},
		{"----","'I've never heard of a.f.d...'/'Still tend to lurk'"},
		{"---","'I haven't even posted yet, I'm lurking'/'A squirrel tried to horde me the other day'"},
		{"--","'They're still responding to my welcome post'/'It can be said I have appeal (a peel)'"},
		{"-","'My name is known, but that's about it'/'Haven't fallen off the tree yet'"},
		{"*","'I got flamed off the newsgroup for yelling at Shim ;)'"},
		{"^","'Have joined, left, returned (repeated as necessary)'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Fruitiness: ");
	if (!strlen(tag))
		strcat(out,"'Old fruit people know my name and what I'm like'/'Squeeze me!  I'm ripe!'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCLHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"+++","'Sorry, Mr. Battleship, I didn't see you!'"},
		{"++","'Can you say \"jungle gym\"?'"},
		{"+","'They look up to me.  Literally.'"},
		{"---","'Please, don't sneeze ...'"},
		{"--","'Please, don't step on *me* ...'"},
		{"-","'Please, don't step on the tail ..."},
		{"*","'Whatever suits me at the moment'"},
		{"^","'Long enough to reach the ground'"},
		{"",""},
		{"",""}};
	int ret;
	unsigned int pos;
	char ctemp[2];

	strcat(out,"Length: ");
	if (!strlen(tag))
		strcat(out,"'About as normal as dragons get.'");
	else
	{
		if (isdigit(tag[0]))
		{
			pos=0;
			ctemp[1]=0;
			while(pos<strlen(tag)&&isdigit(tag[pos]))
			{
				ctemp[0]=tag[pos];
				strcat(out,ctemp);
				pos++;
			}
			switch (tag[pos])
			{
			case 'f':
				strcat(out," feet");
				break;
			case 'm':
				strcat(out," meters");
				break;
			case 'c':
				strcat(out," centimeters");
				break;
			case 'k':
				strcat(out," kilometers");
				break;
			default:
				strcat(out," <Unrecognized modifier>");
				break;
			}
		} else {
			ret=Dictionary(ft,tag,1);
			if (strlen(ft[ret].key))
				strcat(out,ft[ret].out);
			else
				strcat(out,"<Unrecognized modifier>");
		}
	}
}

void DCBHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"A","Acid"},
		{"C","Cold/frost"},
		{"E","Enchantment"},
		{"F","Flame/fire"},
		{"I","Ice"},
		{"L","Lava/magma"},
		{"P","Plasma"},
		{"R","Rot"},
		{"W","Wind"},
		{"Sm","Smoke"},
		{"St","Steam"},
		{"Su","Sulplur"},
		{"Z","Lightning/Electricity"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Breath Weapon: ");
	ret=Dictionary(ft,tag,1);
	if (strlen(ft[ret].key))
		strcat(out,ft[ret].out);
	else
		strcat(out,"<Unrecognized modifier>");
}

void DCEHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"++++","Archmage"},
		{"+++","Not quite there..."},
		{"++","Building my own laboratory"},
		{"+","Passed my magic final!"},
		{"---","The hand is quicker than the eye!"},
		{"--","It just fizzles and dies."},
		{"-","'I did it and the neighborhood's still intact!'"},
		{"?","'What's magic?'"},
		{"!","You can call me \"Almighty One!\""},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Magical Experience: ");
	if (!strlen(tag))
		strcat(out,"'Most of my spells work, but who knows when ... oops.'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCGHandler(char tag[], char *out, int misc)
{
	DictionaryEntry ft[]={
		{"+++++","'I AM Athe and my belly is now several galaxies wide ... while I'm only a few hundred feet long <G>.'"},
		{"++++","'When my belly jiggles the orbits of planets shift out of its way.'"},
		{"+++","'Planets have been known to crack in half with my arrival. >:)'"},
		{"++","'My digestion of food has been known to cause earthquakes.'"},
		{"+","'I move by rolling and flying has always been an effort for me.'"},
		{"-----","'I'm made of really long superstrings, ie i'm one dimensional, all length and no width or depth.  Beware, I can even slip in between atoms >:)'"},
		{"----","'Whoah!  Whaddaya mean I look like a long string with wings?'"},
		{"---","'I've been known to slip into small places. >;)'"},
		{"--","'Ever heard of serpentine?'"},
		{"-","'I'm slightly on the slim side ...'"},
		{"",""},
		{"",""}};
	int ret;

	strcat(out,"Body Shape: ");
	if (!strlen(tag))
		strcat(out,"'What can I say ... I'm normal except for a few feasts here or there. >;)'");
	else
	{
		ret=Dictionary(ft,tag,1);
		if (strlen(ft[ret].key))
			strcat(out,ft[ret].out);
		else
			strcat(out,"<Unrecognized modifier>");
	}
}

void DCDecode(char in[], char *out, int forcever)
{
	int i,curtagpos,tagcount;
	unsigned int pos;
	char DC2,inquotes,inbrackets,found;
	char curtag[100];
	char unknowntags[512];
	// Check version
	numhandlers=0;
	if (strnicmp(in,"DC2.",4)==0||forcever==2)
	{
		if (!forcever)
			strcat(out,"Version:\tDragon Code v2.x\n");
		// Setup for DC2
		DC2=1;
		i=0;
		i+=AddHandler("G",0,DC2GHandler);
		i+=AddHandler("L",0,DC2LHandler);
		i+=AddHandler("W",0,DC2SimpleHandler);
		i+=AddHandler("Tc",0,DC2TcHandler);
		i+=AddHandler("T",0,DC2THandler);
		i+=AddHandler("P",0,DC2PHandler);
		i+=AddHandler("Sk",0,DC2SkHandler);
		i+=AddHandler("C",0,DC2CHandler);
		i+=AddHandler("B",0,DC2BHandler);
		i+=AddHandler("Ac",0,DC2AcHandler);
		i+=AddHandler("A",0,DC2AHandler);
		i+=AddHandler("Fr",0,DC2FrHandler);
		i+=AddHandler("N",0,DC2NHandler);
		i+=AddHandler("M",0,DC2MHandler);
		i+=AddHandler("O",0,DC2OHandler);
		i+=AddHandler("H",1,DC2SimpleHandler);
		i+=AddHandler("$",2,DC2SimpleHandler);
		i+=AddHandler("F",0,DC2FHandler);
		i+=AddHandler("R",0,DC2RHandler);
		//i+=AddHandler("Ac",0,DC2AcHandler);
		i+=AddHandler("J",0,DC2JHandler);
		i+=AddHandler("S",3,DC2SimpleHandler);
		i+=AddHandler("U",0,DC2UHandler);
		i+=AddHandler("I",0,DC2IHandler);
		i+=AddHandler("V",0,DC2VHandler);
		i+=AddHandler("Q",0,DC2QHandler);
		//i+=AddHandler("Tc",0,DC2TcHandler);
		i+=AddHandler("E",0,DC2EHandler);
		i+=AddHandler("Df",4,DC2SimpleHandler);
		if (i!=0)
		{
			strcat(out,"Error creating handlers\n");
			return;
		}
		if (!forcever)
			pos=4;
		else
			pos=0;
	} else if (strnicmp(in,"DC.",3)==0||forcever==1) {
		// Setup for original DC
		if (!forcever)
			strcat(out,"Version:\tDragon Code v1.0\n");
		DC2=0;
		i=0;
		i+=AddHandler("Fr",0,DCFrHandler);
		i+=AddHandler("F",0,DCFHandler);
		i+=AddHandler("S",0,DCSHandler);
		i+=AddHandler("Df",0,DCDfHandler);
		i+=AddHandler("H",0,DCHHandler);
		i+=AddHandler("C",0,DCCHandler);
		i+=AddHandler("A",0,DCAHandler);
		i+=AddHandler("$",0,DCDollarHandler);
		i+=AddHandler("M",0,DCMHandler);
		i+=AddHandler("(M)",0,DCM2Handler);
		i+=AddHandler("D",0,DCDHandler);
		i+=AddHandler("(D)",0,DCD2Handler);
		i+=AddHandler("Wl",0,DCWlHandler);
		//i+=AddHandler("Fr",0,DCFrHandler);
		i+=AddHandler("L",0,DCLHandler);
		i+=AddHandler("B",0,DCBHandler);
		i+=AddHandler("E",0,DCEHandler);
		i+=AddHandler("G",0,DCGHandler);
		if (i!=0)
		{
			strcat(out,"Error creating handlers\n");
			return;
		}
		if (!forcever)
			pos=3;
		else
			pos=0;
	} else {
		strcat(out,"Unrecognized code\n");
		return;
	}

	tagcount=0;
	strcpy(unknowntags,"");
	while(pos<strlen(in))
	{
		// Get the current tag
		curtagpos=0;
		inquotes=0;
		inbrackets=0;
		if (tagcount==0&&in[pos]==' ') tagcount++;			// No species tag
		while (pos<strlen(in)&&in[pos]==' ') pos++;
		if (pos<strlen(in))
		{
			if (DC2)
			{
				while (pos<strlen(in)&&
					((!isupper(in[pos])&&in[pos]!='$'&&in[pos]!=' ')||
					inquotes||inbrackets||(curtagpos==0&&!forcever)||
					(in[pos]==' '&&curtag[0]=='M'&&(in[pos+1]=='+'||in[pos+1]=='-'))||
					(tagcount==0&&!isalnum(in[pos-1])&&in[pos-1]!=' ')))		// OMG!
				{
					if (in[pos]=='\"')
						inquotes=!inquotes;
					if (in[pos]=='['&&!inquotes)
						inbrackets++;
					if (in[pos]==']'&&!inquotes)
						inbrackets--;
					curtag[curtagpos++]=in[pos++];
				}
			} else {
				while (pos<strlen(in)&&(in[pos]!=' '||inquotes||
					((curtag[0]=='m'||curtag[0]=='M')&&in[pos]==' '&&in[pos+1]=='+')))
				{
					if (in[pos]=='(')
						inquotes=1;
					if (in[pos]==')')
						inquotes=0;
					curtag[curtagpos++]=in[pos++];
				}
			}
			curtag[curtagpos]=0;
			if (tagcount==0&&DC2&&!forcever)
				DC2Species(curtag,out);
			else if (tagcount==0&&!forcever)
				DCSpecies(curtag,out);
			else
			{
				// Find the handler
				found=0;
				for (i=0;i<numhandlers;i++)
				{
					if (strnicmp(handlers[i].tag,curtag,strlen(handlers[i].tag))==0)
					{
						// Found the handler
						strcat(out,curtag);
						strcat(out,"\t");
						handlers[i].handler(&curtag[strlen(handlers[i].tag)],
							out,
							handlers[i].misc);
						found=1;
						break;
					}
				}
				if (found)
					strcat(out,"\n");
				if (!found)
				{
					strcat(unknowntags,",");
					strcat(unknowntags,curtag);
				}
			}
			tagcount++;
		}
	}
	if (strlen(unknowntags))
	{
		strcat(out,"Unknown tags: ");
		strcat(out,&unknowntags[1]);
		strcat(out,"\n");
	}
}
